using OpenTK.Mathematics;
using Dashboard.Controls;
using System.Reflection.Metadata;

namespace Dashboard.Layout
{
    /// <summary>
    /// Control attributes for grid layout.
    /// </summary>
    public class GridLayoutAttribute
    {
        /// <summary>
        /// An anchor will keep the relative distance of a control's edges the same during resizes.
        /// </summary>
        /// <remarks><see cref="Dock"/> has higher precedence.</remarks>
        public Anchor Anchor { get; set; } = Anchor.Left | Anchor.Top;

        /// <summary>
        /// Dock will strongly attach a control to its container or its edges.
        /// </summary>
        /// <remarks>Has more precedence than <see cref="Anchor"/></remarks>
        public Dock Dock { get; set; } = Dock.None;

        public void Evaluate(
            ResizedEventArgs parentResizeEvent,
            in Rectangle oldBounds,
            out Rectangle newBounds
            )
        {
            switch (Dock)
            {
            default:
            case Dock.None:
                break;
            case Dock.Top:
                newBounds = new Rectangle(
                    parentResizeEvent.NewSize.X,
                    oldBounds.Size.Y,
                    0,
                    0
                );
                return;
            case Dock.Bottom:
                newBounds = new Rectangle(
                    parentResizeEvent.NewSize.X,
                    parentResizeEvent.NewSize.Y,
                    0,
                    oldBounds.Size.Y
                );
                return;
            case Dock.Left:
                newBounds = new Rectangle(
                    oldBounds.Size.X,
                    parentResizeEvent.NewSize.Y,
                    0,
                    0
                );
                return;
            case Dock.Right:
                newBounds = new Rectangle(
                    parentResizeEvent.NewSize.X,
                    parentResizeEvent.NewSize.Y,
                    parentResizeEvent.NewSize.Y - oldBounds.Size.Y,
                    0
                );
                return;
            }

            Vector2 scale = parentResizeEvent.NewSize / parentResizeEvent.OldSize;

            newBounds = oldBounds;

            if (Anchor.HasFlag(Anchor.Top))
            {
                newBounds.Top = scale.Y * oldBounds.Top;
            }
            
            if (Anchor.HasFlag(Anchor.Left))
            {
                newBounds.Left = scale.X * oldBounds.Left;
            }

            if (Anchor.HasFlag(Anchor.Bottom))
            {
                float margin = scale.Y * (parentResizeEvent.OldSize.Y - newBounds.Bottom);
                newBounds.Bottom = parentResizeEvent.NewSize.Y - margin;
            }

            if (Anchor.HasFlag(Anchor.Right))
            {
                float margin = scale.X * (parentResizeEvent.OldSize.X - newBounds.Right);
                newBounds.Right = parentResizeEvent.NewSize.X - margin;
            }
        }

        /// <summary>
        /// Get the grid layout attribute associated with the given UI-base.
        /// </summary>
        /// <param name="uiBase">The UI-base to query.</param>
        /// <returns>It's grid layout attribute, if any.</returns>
        public static GridLayoutAttribute GetGridLayout(UIBase uiBase)
        {
            const string GRID_LAYOUT_ATTRIBUTE_KEY = "486ddf8c-b75f-4ad4-a51d-5ba20db9bd0e";

            if (
                !uiBase.Attributes.TryGetValue(GRID_LAYOUT_ATTRIBUTE_KEY, out object? attribute)
                || attribute is not GridLayoutAttribute)
            {
                attribute = new GridLayoutAttribute();
                uiBase.Attributes[GRID_LAYOUT_ATTRIBUTE_KEY] = attribute;
            }

            return (GridLayoutAttribute)attribute;
        }
    }
}